@@ -1,7 +1,8 @@
 from helper import unittest, PillowTestCase
 
-from PIL import Image
+from PIL import Image, IcnsImagePlugin
 
+import io
 import sys
 
 # sample icon file
@@ -16,13 +17,15 @@ class TestFileIcns(PillowTestCase):
         # Loading this icon by default should result in the largest size
         # (512x512@2x) being loaded
         im = Image.open(TEST_FILE)
-        im.load()
+
+        # Assert that there is no unclosed file warning
+        self.assert_warning(None, im.load)
+
         self.assertEqual(im.mode, "RGBA")
         self.assertEqual(im.size, (1024, 1024))
         self.assertEqual(im.format, "ICNS")
 
-    @unittest.skipIf(sys.platform != 'darwin',
-                     "requires MacOS")
+    @unittest.skipIf(sys.platform != 'darwin', "requires macOS")
     def test_save(self):
         im = Image.open(TEST_FILE)
 
@@ -35,6 +38,22 @@ class TestFileIcns(PillowTestCase):
         self.assertEqual(reread.size, (1024, 1024))
         self.assertEqual(reread.format, "ICNS")
 
+    @unittest.skipIf(sys.platform != 'darwin', "requires macOS")
+    def test_save_append_images(self):
+        im = Image.open(TEST_FILE)
+
+        temp_file = self.tempfile("temp.icns")
+        provided_im = Image.new('RGBA', (32, 32), (255, 0, 0, 128))
+        im.save(temp_file, append_images=[provided_im])
+
+        reread = Image.open(temp_file)
+        self.assert_image_similar(reread, im, 1)
+
+        reread = Image.open(temp_file)
+        reread.size = (16, 16, 2)
+        reread.load()
+        self.assert_image_equal(reread, provided_im)
+
     def test_sizes(self):
         # Check that we can load all of the sizes, and that the final pixel
         # dimensions are as expected
@@ -48,6 +67,10 @@ class TestFileIcns(PillowTestCase):
             self.assertEqual(im2.mode, 'RGBA')
             self.assertEqual(im2.size, (wr, hr))
 
+        # Check that we cannot load an incorrect size
+        with self.assertRaises(ValueError):
+            im.size = (1, 1)
+
     def test_older_icon(self):
         # This icon was made with Icon Composer rather than iconutil; it still
         # uses PNG rather than JP2, however (since it was made on 10.9).
@@ -82,6 +105,23 @@ class TestFileIcns(PillowTestCase):
             self.assertEqual(im2.mode, 'RGBA')
             self.assertEqual(im2.size, (wr, hr))
 
+    def test_getimage(self):
+        with open(TEST_FILE, 'rb') as fp:
+            icns_file = IcnsImagePlugin.IcnsFile(fp)
+
+            im = icns_file.getimage()
+            self.assertEqual(im.mode, "RGBA")
+            self.assertEqual(im.size, (1024, 1024))
+
+            im = icns_file.getimage((512, 512))
+            self.assertEqual(im.mode, "RGBA")
+            self.assertEqual(im.size, (512, 512))
+
+    def test_not_an_icns_file(self):
+        with io.BytesIO(b'invalid\n') as fp:
+            self.assertRaises(SyntaxError,
+                              IcnsImagePlugin.IcnsFile, fp)
+
 
 if __name__ == '__main__':
     unittest.main()
