@@ -3,7 +3,6 @@ from helper import unittest, PillowTestCase, hopper
 import io
 from PIL import Image, IcoImagePlugin
 
-# sample ppm stream
 TEST_ICO_FILE = "Tests/images/hopper.ico"
 
 
@@ -19,7 +18,7 @@ class TestFileIco(PillowTestCase):
     def test_invalid_file(self):
         with open("Tests/images/flower.jpg", "rb") as fp:
             self.assertRaises(SyntaxError,
-                              lambda: IcoImagePlugin.IcoImageFile(fp))
+                              IcoImagePlugin.IcoImageFile, fp)
 
     def test_save_to_bytes(self):
         output = io.BytesIO()
@@ -29,7 +28,7 @@ class TestFileIco(PillowTestCase):
         # the default image
         output.seek(0)
         reloaded = Image.open(output)
-        self.assertEqual(reloaded.info['sizes'], set([(32, 32), (64, 64)]))
+        self.assertEqual(reloaded.info['sizes'], {(32, 32), (64, 64)})
 
         self.assertEqual(im.mode, reloaded.mode)
         self.assertEqual((64, 64), reloaded.size)
@@ -48,6 +47,11 @@ class TestFileIco(PillowTestCase):
         self.assert_image_equal(reloaded,
                                 hopper().resize((32, 32), Image.LANCZOS))
 
+    def test_incorrect_size(self):
+        im = Image.open(TEST_ICO_FILE)
+        with self.assertRaises(ValueError):
+            im.size = (1, 1)
+
     def test_save_256x256(self):
         """Issue #2264 https://github.com/python-pillow/Pillow/issues/2264"""
         # Arrange
@@ -77,7 +81,7 @@ class TestFileIco(PillowTestCase):
         # Assert
         self.assertEqual(
             im_saved.info['sizes'],
-            set([(16, 16), (24, 24), (32, 32), (48, 48)]))
+            {(16, 16), (24, 24), (32, 32), (48, 48)})
 
 
 if __name__ == '__main__':
