@@ -1,11 +1,7 @@
-from helper import unittest, PillowTestCase, hopper
-
-from PIL import Image
-from PIL import ImageColor
-from PIL import ImageDraw
 import os.path
 
-import sys
+from helper import PillowTestCase, hopper, unittest
+from PIL import Image, ImageColor, ImageDraw
 
 BLACK = (0, 0, 0)
 WHITE = (255, 255, 255)
@@ -30,6 +26,8 @@ BBOX2 = [X0, Y0, X1, Y1]
 POINTS1 = [(10, 10), (20, 40), (30, 30)]
 POINTS2 = [10, 10, 20, 40, 30, 30]
 
+KITE_POINTS = [(10, 50), (70, 10), (90, 50), (70, 90), (10, 50)]
+
 
 class TestImageDraw(PillowTestCase):
 
@@ -48,14 +46,12 @@ class TestImageDraw(PillowTestCase):
         im = Image.open("Tests/images/chi.gif")
 
         draw = ImageDraw.Draw(im)
-        draw.line(((0, 0)), fill=(0, 0, 0))
-        del draw
+        draw.line((0, 0), fill=(0, 0, 0))
 
     def test_mode_mismatch(self):
         im = hopper("RGB").copy()
 
-        self.assertRaises(ValueError,
-                          lambda: ImageDraw.ImageDraw(im, mode="L"))
+        self.assertRaises(ValueError, ImageDraw.ImageDraw, im, mode="L")
 
     def helper_arc(self, bbox, start, end):
         # Arrange
@@ -64,7 +60,6 @@ class TestImageDraw(PillowTestCase):
 
         # Act
         draw.arc(bbox, start, end)
-        del draw
 
         # Assert
         self.assert_image_similar(
@@ -78,6 +73,59 @@ class TestImageDraw(PillowTestCase):
         self.helper_arc(BBOX2, 0, 180)
         self.helper_arc(BBOX2, 0.5, 180.4)
 
+    def test_arc_end_le_start(self):
+        # Arrange
+        im = Image.new("RGB", (W, H))
+        draw = ImageDraw.Draw(im)
+        start = 270.5
+        end = 0
+
+        # Act
+        draw.arc(BBOX1, start=start, end=end)
+
+        # Assert
+        self.assert_image_equal(
+            im, Image.open("Tests/images/imagedraw_arc_end_le_start.png"))
+
+    def test_arc_no_loops(self):
+        # No need to go in loops
+        # Arrange
+        im = Image.new("RGB", (W, H))
+        draw = ImageDraw.Draw(im)
+        start = 5
+        end = 370
+
+        # Act
+        draw.arc(BBOX1, start=start, end=end)
+
+        # Assert
+        self.assert_image_similar(
+            im, Image.open("Tests/images/imagedraw_arc_no_loops.png"), 1)
+
+    def test_arc_width(self):
+        # Arrange
+        im = Image.new("RGB", (W, H))
+        draw = ImageDraw.Draw(im)
+        expected = "Tests/images/imagedraw_arc_width.png"
+
+        # Act
+        draw.arc(BBOX1, 10, 260, width=5)
+
+        # Assert
+        self.assert_image_similar(im, Image.open(expected), 1)
+
+    def test_arc_width_fill(self):
+        # Arrange
+        im = Image.new("RGB", (W, H))
+        draw = ImageDraw.Draw(im)
+        expected = "Tests/images/imagedraw_arc_width_fill.png"
+
+        # Act
+        draw.arc(BBOX1, 10, 260, fill="yellow", width=5)
+
+        # Assert
+        self.assert_image_similar(im, Image.open(expected), 1)
+
     def test_bitmap(self):
         # Arrange
         small = Image.open("Tests/images/pil123rgba.png").resize((50, 50))
@@ -86,51 +134,76 @@ class TestImageDraw(PillowTestCase):
 
         # Act
         draw.bitmap((10, 10), small)
-        del draw
 
         # Assert
         self.assert_image_equal(
             im, Image.open("Tests/images/imagedraw_bitmap.png"))
 
-    def helper_chord(self, bbox, start, end):
+    def helper_chord(self, mode, bbox, start, end):
         # Arrange
-        im = Image.new("RGB", (W, H))
+        im = Image.new(mode, (W, H))
         draw = ImageDraw.Draw(im)
+        expected = "Tests/images/imagedraw_chord_{}.png".format(mode)
 
         # Act
         draw.chord(bbox, start, end, fill="red", outline="yellow")
-        del draw
 
         # Assert
-        self.assert_image_similar(
-            im, Image.open("Tests/images/imagedraw_chord.png"), 1)
+        self.assert_image_similar(im, Image.open(expected), 1)
 
     def test_chord1(self):
-        self.helper_chord(BBOX1, 0, 180)
-        self.helper_chord(BBOX1, 0.5, 180.4)
+        for mode in ["RGB", "L"]:
+            self.helper_chord(mode, BBOX1, 0, 180)
+            self.helper_chord(mode, BBOX1, 0.5, 180.4)
 
     def test_chord2(self):
-        self.helper_chord(BBOX2, 0, 180)
-        self.helper_chord(BBOX2, 0.5, 180.4)
+        for mode in ["RGB", "L"]:
+            self.helper_chord(mode, BBOX2, 0, 180)
+            self.helper_chord(mode, BBOX2, 0.5, 180.4)
 
-    def helper_ellipse(self, bbox):
+    def test_chord_width(self):
         # Arrange
         im = Image.new("RGB", (W, H))
         draw = ImageDraw.Draw(im)
+        expected = "Tests/images/imagedraw_chord_width.png"
+
+        # Act
+        draw.chord(BBOX1, 10, 260, outline="yellow", width=5)
+
+        # Assert
+        self.assert_image_similar(im, Image.open(expected), 1)
+
+    def test_chord_width_fill(self):
+        # Arrange
+        im = Image.new("RGB", (W, H))
+        draw = ImageDraw.Draw(im)
+        expected = "Tests/images/imagedraw_chord_width_fill.png"
+
+        # Act
+        draw.chord(BBOX1, 10, 260, fill="red", outline="yellow", width=5)
+
+        # Assert
+        self.assert_image_similar(im, Image.open(expected), 1)
+
+    def helper_ellipse(self, mode, bbox):
+        # Arrange
+        im = Image.new(mode, (W, H))
+        draw = ImageDraw.Draw(im)
+        expected = "Tests/images/imagedraw_ellipse_{}.png".format(mode)
 
         # Act
         draw.ellipse(bbox, fill="green", outline="blue")
-        del draw
 
         # Assert
-        self.assert_image_similar(
-            im, Image.open("Tests/images/imagedraw_ellipse.png"), 1)
+        self.assert_image_similar(im, Image.open(expected), 1)
 
     def test_ellipse1(self):
-        self.helper_ellipse(BBOX1)
+        for mode in ["RGB", "L"]:
+            self.helper_ellipse(mode, BBOX1)
 
     def test_ellipse2(self):
-        self.helper_ellipse(BBOX2)
+        for mode in ["RGB", "L"]:
+            self.helper_ellipse(mode, BBOX2)
 
     def test_ellipse_edge(self):
         # Arrange
@@ -139,12 +212,45 @@ class TestImageDraw(PillowTestCase):
 
         # Act
         draw.ellipse(((0, 0), (W-1, H)), fill="white")
-        del draw
 
         # Assert
         self.assert_image_similar(
             im, Image.open("Tests/images/imagedraw_ellipse_edge.png"), 1)
 
+    def test_ellipse_symmetric(self):
+        for bbox in [
+            (25, 25, 76, 76),
+            (25, 25, 75, 75)
+        ]:
+            im = Image.new("RGB", (101, 101))
+            draw = ImageDraw.Draw(im)
+            draw.ellipse(bbox, fill="green", outline="blue")
+            self.assert_image_equal(im, im.transpose(Image.FLIP_LEFT_RIGHT))
+
+    def test_ellipse_width(self):
+        # Arrange
+        im = Image.new("RGB", (W, H))
+        draw = ImageDraw.Draw(im)
+        expected = "Tests/images/imagedraw_ellipse_width.png"
+
+        # Act
+        draw.ellipse(BBOX1, outline="blue", width=5)
+
+        # Assert
+        self.assert_image_similar(im, Image.open(expected), 1)
+
+    def test_ellipse_width_fill(self):
+        # Arrange
+        im = Image.new("RGB", (W, H))
+        draw = ImageDraw.Draw(im)
+        expected = "Tests/images/imagedraw_ellipse_width_fill.png"
+
+        # Act
+        draw.ellipse(BBOX1, fill="green", outline="blue", width=5)
+
+        # Assert
+        self.assert_image_similar(im, Image.open(expected), 1)
+
     def helper_line(self, points):
         # Arrange
         im = Image.new("RGB", (W, H))
@@ -152,7 +258,6 @@ class TestImageDraw(PillowTestCase):
 
         # Act
         draw.line(points, fill="yellow", width=2)
-        del draw
 
         # Assert
         self.assert_image_equal(
@@ -180,7 +285,6 @@ class TestImageDraw(PillowTestCase):
         s.line(x0, y0)
 
         draw.shape(s, fill=1)
-        del draw
 
         # Assert
         self.assert_image_equal(
@@ -202,7 +306,6 @@ class TestImageDraw(PillowTestCase):
         s.line(x0, y0)
 
         draw.shape(s, outline="blue")
-        del draw
 
         # Assert
         self.assert_image_equal(
@@ -215,7 +318,6 @@ class TestImageDraw(PillowTestCase):
 
         # Act
         draw.pieslice(bbox, start, end, fill="white", outline="blue")
-        del draw
 
         # Assert
         self.assert_image_similar(
@@ -229,6 +331,30 @@ class TestImageDraw(PillowTestCase):
         self.helper_pieslice(BBOX2, -90, 45)
         self.helper_pieslice(BBOX2, -90.5, 45.4)
 
+    def test_pieslice_width(self):
+        # Arrange
+        im = Image.new("RGB", (W, H))
+        draw = ImageDraw.Draw(im)
+        expected = "Tests/images/imagedraw_pieslice_width.png"
+
+        # Act
+        draw.pieslice(BBOX1, 10, 260, outline="blue", width=5)
+
+        # Assert
+        self.assert_image_similar(im, Image.open(expected), 1)
+
+    def test_pieslice_width_fill(self):
+        # Arrange
+        im = Image.new("RGB", (W, H))
+        draw = ImageDraw.Draw(im)
+        expected = "Tests/images/imagedraw_pieslice_width_fill.png"
+
+        # Act
+        draw.pieslice(BBOX1, 10, 260, fill="white", outline="blue", width=5)
+
+        # Assert
+        self.assert_image_similar(im, Image.open(expected), 1)
+
     def helper_point(self, points):
         # Arrange
         im = Image.new("RGB", (W, H))
@@ -236,7 +362,6 @@ class TestImageDraw(PillowTestCase):
 
         # Act
         draw.point(points, fill="yellow")
-        del draw
 
         # Assert
         self.assert_image_equal(
@@ -255,7 +380,6 @@ class TestImageDraw(PillowTestCase):
 
         # Act
         draw.polygon(points, fill="red", outline="blue")
-        del draw
 
         # Assert
         self.assert_image_equal(
@@ -267,6 +391,22 @@ class TestImageDraw(PillowTestCase):
     def test_polygon2(self):
         self.helper_polygon(POINTS2)
 
+    def test_polygon_kite(self):
+        # Test drawing lines of different gradients (dx>dy, dy>dx) and
+        # vertical (dx==0) and horizontal (dy==0) lines
+        for mode in ["RGB", "L"]:
+            # Arrange
+            im = Image.new(mode, (W, H))
+            draw = ImageDraw.Draw(im)
+            expected = "Tests/images/imagedraw_polygon_kite_{}.png".format(
+                mode)
+
+            # Act
+            draw.polygon(KITE_POINTS, fill="blue", outline="yellow")
+
+            # Assert
+            self.assert_image_equal(im, Image.open(expected))
+
     def helper_rectangle(self, bbox):
         # Arrange
         im = Image.new("RGB", (W, H))
@@ -274,7 +414,6 @@ class TestImageDraw(PillowTestCase):
 
         # Act
         draw.rectangle(bbox, fill="black", outline="green")
-        del draw
 
         # Assert
         self.assert_image_equal(
@@ -286,20 +425,65 @@ class TestImageDraw(PillowTestCase):
     def test_rectangle2(self):
         self.helper_rectangle(BBOX2)
 
-    def test_floodfill(self):
+    def test_big_rectangle(self):
+        # Test drawing a rectangle bigger than the image
+        # Arrange
+        im = Image.new("RGB", (W, H))
+        bbox = [(-1, -1), (W+1, H+1)]
+        draw = ImageDraw.Draw(im)
+        expected = "Tests/images/imagedraw_big_rectangle.png"
+
+        # Act
+        draw.rectangle(bbox, fill="orange")
+
+        # Assert
+        self.assert_image_similar(im, Image.open(expected), 1)
+
+    def test_rectangle_width(self):
         # Arrange
         im = Image.new("RGB", (W, H))
         draw = ImageDraw.Draw(im)
-        draw.rectangle(BBOX2, outline="yellow", fill="green")
-        centre_point = (int(W/2), int(H/2))
-        red = ImageColor.getrgb("red")
-        im_floodfill = Image.open("Tests/images/imagedraw_floodfill.png")
+        expected = "Tests/images/imagedraw_rectangle_width.png"
 
         # Act
-        ImageDraw.floodfill(im, centre_point, red)
+        draw.rectangle(BBOX1, outline="green", width=5)
 
         # Assert
-        self.assert_image_equal(im, im_floodfill)
+        self.assert_image_equal(im, Image.open(expected))
+
+    def test_rectangle_width_fill(self):
+        # Arrange
+        im = Image.new("RGB", (W, H))
+        draw = ImageDraw.Draw(im)
+        expected = "Tests/images/imagedraw_rectangle_width_fill.png"
+
+        # Act
+        draw.rectangle(BBOX1, fill="blue", outline="green", width=5)
+
+        # Assert
+        self.assert_image_equal(im, Image.open(expected))
+
+    def test_floodfill(self):
+        red = ImageColor.getrgb("red")
+
+        for mode, value in [
+            ("L", 1),
+            ("RGBA", (255, 0, 0, 0)),
+            ("RGB", red)
+        ]:
+            # Arrange
+            im = Image.new(mode, (W, H))
+            draw = ImageDraw.Draw(im)
+            draw.rectangle(BBOX2, outline="yellow", fill="green")
+            centre_point = (int(W/2), int(H/2))
+
+            # Act
+            ImageDraw.floodfill(im, centre_point, value)
+
+            # Assert
+            expected = "Tests/images/imagedraw_floodfill_"+mode+".png"
+            im_floodfill = Image.open(expected)
+            self.assert_image_equal(im, im_floodfill)
 
         # Test that using the same colour does not change the image
         ImageDraw.floodfill(im, centre_point, red)
@@ -308,11 +492,12 @@ class TestImageDraw(PillowTestCase):
         # Test that filling outside the image does not change the image
         ImageDraw.floodfill(im, (W, H), red)
         self.assert_image_equal(im, im_floodfill)
-        del draw
 
+        # Test filling at the edge of an image
+        im = Image.new("RGB", (1, 1))
+        ImageDraw.floodfill(im, (0, 0), red)
+        self.assert_image_equal(im, Image.new("RGB", (1, 1), red))
 
-    @unittest.skipIf(hasattr(sys, 'pypy_version_info'),
-                     "Causes fatal RPython error on PyPy")
     def test_floodfill_border(self):
         # floodfill() is experimental
 
@@ -326,7 +511,24 @@ class TestImageDraw(PillowTestCase):
         ImageDraw.floodfill(
             im, centre_point, ImageColor.getrgb("red"),
             border=ImageColor.getrgb("black"))
-        del draw
+
+        # Assert
+        self.assert_image_equal(
+            im, Image.open("Tests/images/imagedraw_floodfill2.png"))
+
+    def test_floodfill_thresh(self):
+        # floodfill() is experimental
+
+        # Arrange
+        im = Image.new("RGB", (W, H))
+        draw = ImageDraw.Draw(im)
+        draw.rectangle(BBOX2, outline="darkgreen", fill="green")
+        centre_point = (int(W/2), int(H/2))
+
+        # Act
+        ImageDraw.floodfill(
+            im, centre_point, ImageColor.getrgb("red"),
+            thresh=30)
 
         # Assert
         self.assert_image_equal(
@@ -341,7 +543,7 @@ class TestImageDraw(PillowTestCase):
             for y in range(0, size[1]):
                 if (x + y) % 2 == 0:
                     img.putpixel((x, y), background2)
-        return (img, ImageDraw.Draw(img))
+        return img, ImageDraw.Draw(img)
 
     def test_square(self):
         expected = Image.open(os.path.join(IMAGES_PATH, 'square.png'))
@@ -479,6 +681,88 @@ class TestImageDraw(PillowTestCase):
         self.assert_image_equal(img, expected,
                                 'line oblique 45 inverted 3px wide B failed')
 
+    def test_wide_line_dot(self):
+        # Test drawing a wide "line" from one point to another just draws
+        # a single point
+        # Arrange
+        im = Image.new("RGB", (W, H))
+        draw = ImageDraw.Draw(im)
+        expected = "Tests/images/imagedraw_wide_line_dot.png"
+
+        # Act
+        draw.line([(50, 50), (50, 50)], width=3)
+
+        # Assert
+        self.assert_image_similar(im, Image.open(expected), 1)
+
+    def test_line_joint(self):
+        im = Image.new("RGB", (500, 325))
+        draw = ImageDraw.Draw(im)
+        expected = "Tests/images/imagedraw_line_joint_curve.png"
+
+        # Act
+        xy = [(400, 280), (380, 280), (450, 280), (440, 120), (350, 200),
+              (310, 280), (300, 280), (250, 280), (250, 200), (150, 200),
+              (150, 260), (50, 200), (150, 50), (250, 100)]
+        draw.line(xy, GRAY, 50, "curve")
+
+        # Assert
+        self.assert_image_similar(im, Image.open(expected), 3)
+
+    def test_textsize_empty_string(self):
+        # https://github.com/python-pillow/Pillow/issues/2783
+        # Arrange
+        im = Image.new("RGB", (W, H))
+        draw = ImageDraw.Draw(im)
+
+        # Act
+        # Should not cause 'SystemError: <built-in method getsize of
+        # ImagingFont object at 0x...> returned NULL without setting an error'
+        draw.textsize("")
+        draw.textsize("\n")
+        draw.textsize("test\n")
+
+    def test_same_color_outline(self):
+        # Prepare shape
+        x0, y0 = 5, 5
+        x1, y1 = 5, 50
+        x2, y2 = 95, 50
+        x3, y3 = 95, 5
+
+        s = ImageDraw.Outline()
+        s.move(x0, y0)
+        s.curve(x1, y1, x2, y2, x3, y3)
+        s.line(x0, y0)
+
+        # Begin
+        for mode in ["RGB", "L"]:
+            for fill, outline in [
+                ["red", None],
+                ["red", "red"],
+                ["red", "#f00"]
+            ]:
+                for operation, args in {
+                    'chord': [BBOX1, 0, 180],
+                    'ellipse': [BBOX1],
+                    'shape': [s],
+                    'pieslice': [BBOX1, -90, 45],
+                    'polygon': [[(18, 30), (85, 30), (60, 72)]],
+                    'rectangle': [BBOX1]
+                }.items():
+                    # Arrange
+                    im = Image.new(mode, (W, H))
+                    draw = ImageDraw.Draw(im)
+
+                    # Act
+                    draw_method = getattr(draw, operation)
+                    args += [fill, outline]
+                    draw_method(*args)
+
+                    # Assert
+                    expected = ("Tests/images/imagedraw_outline"
+                                "_{}_{}.png".format(operation, mode))
+                    self.assert_image_similar(im, Image.open(expected), 1)
+
 
 if __name__ == '__main__':
     unittest.main()
